import { GroupService } from './../../../group/services/group.service';
import { Assignment } from './../../interfaces/assignment.interface';
import { Component, OnInit, ChangeDetectorRef } from '@angular/core';
import { FormGroup, FormControl, Validators, FormArray } from '@angular/forms';
import { SubjectService } from '../../services/subject.service';
import { ActivatedRoute, Router } from '@angular/router';
import { Subject } from '../../interfaces/subject.interface';
import { Test } from '../../interfaces/test.interface';
import { Comment } from '../../interfaces/comment.interface';

@Component({
  selector: 'app-subject-edit',
  templateUrl: './subject-edit.component.html',
  styleUrls: ['./subject-edit.component.scss']
})
export class SubjectEditComponent implements OnInit {

  subject: Subject;
  subjectForm: FormGroup;
  subjectId: number;
  submited = false;
  groupId: number;
  editable = true;

  constructor(private activatedRoute: ActivatedRoute,
              private subjectService: SubjectService,
              private groupService: GroupService,
              private router: Router,
              private cd: ChangeDetectorRef) { }

  ngOnInit(): void {
    this.groupId = this.activatedRoute.snapshot.params.groupId;
    this.subject = this.createEmptySubject();
    this.subjectForm = this.createSubjectForm();
    this.subjectId = this.activatedRoute.snapshot.params.id;
    if (this.subjectId) {
      this.subjectService.getById(this.subjectId, localStorage.getItem('userId')).subscribe((subjectPayload) => {
        this.subject = subjectPayload;
        this.updateSubjectForm(this.subject);
        this.editable = this.subject.editedBy === localStorage.getItem('userId');
        if (!this.editable) {
          this.subjectForm.disable();
        }
      });
    }
  }

  backToGroup(): void {
    if (this.editable && this.subjectId) {
      this.subjectService.unlock(this.subjectId).subscribe(() => {
        this.router.navigateByUrl('/group/edit/' + this.groupId);
      });
    } else {
      this.router.navigateByUrl('/group/edit/' + this.groupId);
    }
  }

  createEmptySubject(): Subject {
    return {
      id: null,
      groupId: this.groupId,
      name: '',
      lectureTeacher: '',
      labTeacher: '',
      mainExam: false,
      examDate: new Date(),
      editedBy: null,
      tests: [],
      assignments: [],
      comments: []
    };
  }

  createSubjectForm(): FormGroup {
    return new FormGroup({
      id: new FormControl(null),
      groupId: new FormControl(this.groupId),
      name: new FormControl('', Validators.required),
      lectureTeacher: new FormControl(''),
      labTeacher: new FormControl(''),
      mainExam: new FormControl(false),
      examDate: new FormControl(this.parseDateTimeToISOFormat(new Date())),
      editedBy: new FormControl(null),
      tests: new FormArray([]),
      assignments: new FormArray([]),
      comments: new FormArray([]),
    });
  }

  updateSubjectForm(subject: Subject): void {
    for (let test of this.subject.tests) {
      (this.subjectForm.controls.tests as FormArray).push(new FormGroup({
        id: new FormControl(null),
        date: new FormControl(null),
        scope: new FormControl('', Validators.required),
        finalMarkPercent: new FormControl(0),
        subjectId: new FormControl(null)
      }));
    }
    for (let assignment of this.subject.assignments) {
      (this.subjectForm.controls.assignments as FormArray).push(new FormGroup({
        id: new FormControl(null),
        name: new FormControl('', Validators.required),
        deadline: new FormControl(null),
        description: new FormControl('', Validators.required),
        finalMarkPercent: new FormControl(0),
        subjectId: new FormControl(null)
      }));
    }
    for (let comment of this.subject.comments) {
      (this.subjectForm.controls.comments as FormArray).push(new FormGroup({
        id: new FormControl(null),
        text: new FormControl('', Validators.required),
        subjectId: new FormControl(null)
      }));
    }
    this.subjectForm.setValue(subject);
  }

  onAddNewTest(test?: Test): void {
    if (!test) {
      test = {
        date: new Date(),
        scope: '',
        finalMarkPercent: 0
      };
    }
    (this.subjectForm.controls.tests as FormArray).push(new FormGroup({
      date: new FormControl(this.parseDateTimeToISOFormat(test.date)),
      scope: new FormControl(test.scope),
      finalMarkPercent: new FormControl(test.finalMarkPercent)
    }));
  }

  onAddNewAssignment(assignment?: Assignment): void {
    if (!assignment) {
      assignment = {
        name: '',
        deadline: new Date(),
        description: '',
        finalMarkPercent: 0
      };
    }
    (this.subjectForm.controls.assignments as FormArray).push(new FormGroup({
      name: new FormControl(assignment.name, Validators.required),
      deadline: new FormControl(this.parseDateTimeToISOFormat(assignment.deadline)),
      description: new FormControl(assignment.description, Validators.required),
      finalMarkPercent: new FormControl(assignment.finalMarkPercent)
    }));
  }

  onAddNewComment(comment?: Comment): void {
    if (!comment) {
      comment = {
        text: ''
      };
    }
    (this.subjectForm.controls.comments as FormArray).push(new FormGroup({
      text: new FormControl(comment.text)
    }));
  }

  onSubjectSave(): void {
    this.submited = true;
    //this.subjectForm.updateValueAndValidity();
    //this.cd.detectChanges();
    if (this.subjectForm.valid) {
      if (this.subjectId) {
        this.subjectService.update(this.subjectForm.value).subscribe(() => {
          this.router.navigateByUrl('/group/edit/' + this.groupId);
        });
      } else {
        this.subjectService.add(this.subjectForm.value).subscribe(() => {
          this.router.navigateByUrl('/group/edit/' + this.groupId);
        });
      }
    }
  }

  private parseDateTimeToISOFormat(date: Date): string {
    return new Date(date.getTime() - (date.getTimezoneOffset() * 60000)).toISOString().slice(0, 16);
  }

}
